// This file is part of noxos and licensed under the MIT open source license

#include "public/nox/memory.h"

void memcpy(void* dest, void* src, uint64_t n) {
    uint32_t  n_64    = n >> 3; // fast division by 8
    uint64_t* src_64  = (uint64_t*)src;
    uint64_t* dest_64 = (uint64_t*)dest;

    for (uint32_t i = 0; i < n_64; i++) {
        dest_64[i] = src_64[i];
    }

    uint32_t n_8    = n % 8;
    uint8_t* src_8  = &((uint8_t*)src)[n - n_8];
    uint8_t* dest_8 = &((uint8_t*)dest)[n - n_8];

    for (uint32_t i = 0; i < n_8; i++) {
        dest_8[i] = src_8[i];
    }
}

void memset(void* dest, uint8_t data, uint64_t n) {
    uint32_t  n_64    = n >> 3; // fast division by 8
    uint64_t* dest_64 = (uint64_t*)dest;
    uint64_t  data_64 = data;

    data_64 |= (data_64 << 8) | (data_64 << 16) | (data_64 << 24) | (data_64 << 32) | (data_64 << 40) | (data_64 << 48) | (data_64 << 56);

    for (uint32_t i = 0; i < n_64; i++) {
        dest_64[i] = data_64;
    }

    uint32_t n_8    = n % 8;
    uint8_t* dest_8 = &((uint8_t*)dest)[n - n_8];

    for (uint32_t i = 0; i < n_8; i++) {
        dest_8[i] = data;
    }
}

bool memcmp(void* a, void* b, uint64_t n) {
    uint32_t  n_64 = n >> 3; // fast division by 8
    uint64_t* a_64 = (uint64_t*)a;
    uint64_t* b_64 = (uint64_t*)b;

    for (uint32_t i = 0; i < n_64; i++) {
        if (a_64[i] != b_64[i]) {
            return false;
        }
    }

    uint32_t n_8 = n % 8;
    uint8_t* a_8 = &((uint8_t*)a)[n - n_8];
    uint8_t* b_8 = &((uint8_t*)b)[n - n_8];

    for (uint32_t i = 0; i < n_8; i++) {
        if (a_8[i] != b_8[i]) {
            return false;
        }
    }

    return true;
}

void* malloc(uint64_t size) {

}

void free(void* addr) {

}
